/*
  Moves color gradient in reciprocating motion with adjustable settings.

  Example ready to use with ESP-8266.
  To use with other boards, replace pin numbers accordingly.

  Before uploading, replace <YOUR WIFI NAME> and <YOUR WIFI PASSWORD>
  with your WiFi credentials.
*/

#include <Adafruit_NeoPixel.h>
#include <WiFiGateDevice.h>

WiFiGateDevice device("<YOUR WIFI NAME>", "<YOUR WIFI PASSWORD>");
Adafruit_NeoPixel strip(30, D2, NEO_GRB + NEO_KHZ800);

// frame rate: 20fps => 50ms interval between frames
const int FRAME_INTERVAL = 50;

// adjustable settings
GateFloat positionIncrement = device.factory.createFloat("input");
GateInt brightness = device.factory.createInt("input");
GateFloat gradient = device.factory.createFloat("input");

// dynamic parameters
unsigned long nextFrame = 0;
double markerPosition = 0;
int direction = 1;

void setup() {
  strip.begin();
  strip.show();

  device.setName("LED controller");

  positionIncrement.setName("Speed");
  positionIncrement.setMin(0.005);
  positionIncrement.setMax(0.1);

  brightness.setName("Brightness");
  brightness.setMin(1);
  brightness.setMax(100);
  brightness.setValue(50);

  gradient.setName("Gradient");
  gradient.setMin(1);
  gradient.setMax(4);

  device.start();
}

void loop() {
  loopStrips();
  device.loop();
}

// displays subsequent frames at given interval
void loopStrips() {
  if (millis() > nextFrame) {
    nextFrame = millis() + FRAME_INTERVAL;
    moveMarker();
    showFrame();
  }
}

// moves "center" of gradient in reciprocating motion
void moveMarker() {
  markerPosition += direction * positionIncrement.getValue();
  if (markerPosition > 1) {
    markerPosition = 1;
    direction = -1;
  } else if (markerPosition < 0) {
    markerPosition = 0;
    direction = 1;
  }
}

// sets colors of individual pixels basing on current marker position
void showFrame() {
  for(int pixel = 0; pixel < strip.numPixels(); pixel++) {
    double pixelPosition = (double) pixel / (strip.numPixels() - 1);
    double colorFactor = getColorFactor(pixelPosition);
    int color1 = (int)(colorFactor * brightness.getValue());
    int color2 = (int)((1 - colorFactor) * brightness.getValue());
    strip.setPixelColor(pixel, strip.Color(color2, color1, 0));
  }
  strip.show();
}


// calculates factor for color calculation (extracted for clarity)
double getColorFactor(double pixelPosition) {
  double distance = markerPosition - pixelPosition;
  double factor = 0.5 + gradient.getValue() * distance;
  if (factor > 1) {
    return 1;
  }
  if (factor < 0) {
    return 0;
  }
  return factor;
}
