/*
  Moves color gradient in reciprocating motion

  Example ready to use with Arduino Uno.
  To use with other boards, replace pin numbers accordingly.
*/

#include <Adafruit_NeoPixel.h>

Adafruit_NeoPixel strip(30, 3, NEO_GRB + NEO_KHZ800);

// frame rate: 20fps => 50ms interval between frames
const int FRAME_INTERVAL = 50;

// static settings
double positionIncrement = 0.04;
int brightness = 100;
double gradient = 2;

// dynamic parameters
unsigned long nextFrame = 0;
double markerPosition = 0;
int direction = 1;

void setup() {
  strip.begin();
  strip.show();
}

void loop() {
  loopStrips();
}

// displays subsequent frames at given interval
void loopStrips() {
  if (millis() > nextFrame) {
    nextFrame = millis() + FRAME_INTERVAL;
    moveMarker();
    showFrame();
  }
}

// moves "center" of gradient in reciprocating motion
void moveMarker() {
  markerPosition += direction * positionIncrement;
  if (markerPosition > 1) {
    markerPosition = 1;
    direction = -1;
  } else if (markerPosition < 0) {
    markerPosition = 0;
    direction = 1;
  }
}

// sets colors of individual pixels basing on current marker position
void showFrame() {
  for(int pixel = 0; pixel < strip.numPixels(); pixel++) {
    double pixelPosition = (double) pixel / (strip.numPixels() - 1);
    double colorFactor = getColorFactor(pixelPosition);
    int color1 = (int)(colorFactor * brightness);
    int color2 = (int)((1 - colorFactor) * brightness);
    strip.setPixelColor(pixel, strip.Color(color2, color1, 0));
  }
  strip.show();
}


// calculates factor for color calculation (extracted for clarity)
double getColorFactor(double pixelPosition) {
  double distance = markerPosition - pixelPosition;
  double factor = 0.5 + gradient * distance;
  if (factor > 1) {
    return 1;
  }
  if (factor < 0) {
    return 0;
  }
  return factor;
}
